#!/usr/bin/env ruby
#
# Amarok Script for fetching genre information from Last.fm and writing id3tag
# version: 0.3.3
# (c) 2008 Brian Astrup Mikkelsen <brianam@es.aau.dk>
#
# License: GNU General Public License V2
require 'net/http'
require 'rexml/document'
require 'uri'
begin 
	require 'taglib'
rescue LoadError
	error = "ruby-taglib is required to run last.fm genre tagger"
	`dcop amarok playlist popupMessage "#{error}"`
	exit
end

def xml_genre_artist(artist)
	url =  URI.escape("http://ws.audioscrobbler.com/1.0/artist/#{artist}/toptags.xml")
end

def xml_genre_album(artist,album)
	url = URI.escape("http://ws.audioscrobbler.com/1.0/album/#{artist}/#{album}/toptags.xml")
end

def xml_genre_title(artist,title)
	url = URI.escape("http://ws.audioscrobbler.com/1.0/track/#{artist}/#{title}/toptags.xml")
end

def get_genre_xml(url)
	begin
		uri = URI.parse(url)
	rescue
		return FALSE
	end
	begin 
		xml_data = Net::HTTP.get_response(uri).body
	rescue
		return FALSE
	end
	begin
		doc = REXML::Document.new(xml_data)
	rescue
		return FALSE
	end
	genre = []
	doc.elements.each('toptags/tag/name') do | tag |
		genre << tag.text
	end
	return genre
end

def get_genre(artist,album,title)
	url = []
	url << xml_genre_title(artist,title) 
	url << xml_genre_album(artist,album)
	url << xml_genre_artist(artist)
	nodata = "No data recieved for: #{artist} -  #{title}"
	
	arrays = url.size-1
	(0..arrays).each do |array|
		if genre = get_genre_xml(url[array]) then 
			genre.each do |current|
				if (not blacklisted(current)) then
					return current
				end
			end
		end
	end
	`dcop amarok playlist shortStatusMessage "#{nodata}"`
	return FALSE
end

def blacklisted(tag)
	tag = tag.downcase
	blacklist = get_blacklist()

	if blacklist.index(tag)
		return TRUE
	else
		return FALSE
	end
end

def get_blacklist()
	home = ENV['HOME']
	blackfile = home+"/.kde/share/apps/amarok/scripts-data/genre-tagger.csv"

	if not File.exist?(blackfile) then 
		init_blacklist()
	end

	file = File.new(blackfile,"r");
	data = ""
	while (line = file.gets) do
		data=data+line
	end
	data = data.split(",")
	return data
end

def init_blacklist()
	blacklist = ["albums i own","seen live","overrated"]
	str = blacklist.join(',')
	write_blacklist(str)
end

def edit_blacklist()
	data = get_blacklist()
	data = data.join("\n")
	data = `kdialog --textinputbox "Blacklist" "#{data}"`
	if data != "" then
		data = data.gsub("\n",",")
		write_blacklist(data)
	end
end

def write_blacklist(data)
	home = ENV['HOME']
	blackfile = home+"/.kde/share/apps/amarok/scripts-data/genre-tagger.csv"
	fp = File.new(blackfile,"w");
	fp.puts(data)
	fp.close
end

def capitalize(genre)
	arr = genre.split(" ")
	arr = arr.collect{ |value| value.capitalize }
	str = arr.join(' ')
	return str
end

def write_id3(file,genre)
	genre = capitalize(genre)
	tag = TagLib::File.new(file)
	if genre != tag.genre then 
		tag.genre=(genre)
		tag.save
		`dcop amarok playlist shortStatusMessage "id3tag updated - genre: #{genre}"`
	end
	tag.close
end

########## MAIN ###########

loop do
    message = gets().chomp()
    command = /[A-Za-z]*/.match( message ).to_s()

    case command
        when "configure"
			edit_blacklist()
		
        when "trackChange"
			artist = `dcop amarok player artist`.chomp
			album = `dcop amarok player album`.chomp
			title = `dcop amarok player title`.chomp
			file = `dcop amarok player path`.chomp
			if genre = get_genre(artist,album,title) then
				write_id3(file,genre)
			end
    end
end

